@@ -363,10 +363,7 @@ static void
 size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
 {
     GeglGtkView *self = GEGL_GTK_VIEW(widget);
-    gint scale_factor;
-
-    scale_factor = gtk_widget_get_scale_factor (widget);
-    view_helper_set_allocation(GET_PRIVATE(self), allocation, scale_factor);
+    view_helper_set_allocation(GET_PRIVATE(self), allocation);
 }
 
 static void
