@@ -43,7 +43,6 @@
 #include "photos-gom-miner.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
-#include "photos-mode-controller.h"
 #include "photos-properties-dialog.h"
 #include "photos-query.h"
 #include "photos-resources.h"
@@ -61,6 +60,7 @@
 
 struct _PhotosApplicationPrivate
 {
+  GCancellable *create_window_cancellable;
   GList *miners;
   GList *miners_running;
   GResource *resource;
@@ -68,6 +68,8 @@ struct _PhotosApplicationPrivate
   GSettings *ss_settings;
   GSimpleAction *fs_action;
   GSimpleAction *gear_action;
+  GSimpleAction *load_next_action;
+  GSimpleAction *load_previous_action;
   GSimpleAction *open_action;
   GSimpleAction *print_action;
   GSimpleAction *properties_action;
@@ -79,14 +81,15 @@ struct _PhotosApplicationPrivate
   GSimpleAction *sel_none_action;
   GSimpleAction *set_bg_action;
   GSimpleAction *set_ss_action;
-  GSimpleAction *remote_display_action;
   GtkWidget *main_window;
   PhotosCameraCache *camera_cache;
-  PhotosModeController *mode_cntrlr;
   PhotosSearchContextState *state;
   PhotosSearchProvider *search_provider;
   TrackerExtractPriority *extract_priority;
+  guint create_miners_count;
   guint32 activation_timestamp;
+  gulong source_added_id;
+  gulong source_removed_id;
 };
 
 enum
@@ -119,8 +122,16 @@ static const gchar *DESKTOP_KEY_COLOR_SHADING_TYPE = "color-shading-type";
 static const gchar *DESKTOP_KEY_PRIMARY_COLOR = "primary-color";
 static const gchar *DESKTOP_KEY_SECONDARY_COLOR = "secondary-color";
 
+typedef struct _PhotosApplicationCreateData PhotosApplicationCreateData;
 typedef struct _PhotosApplicationRefreshData PhotosApplicationRefreshData;
 
+struct _PhotosApplicationCreateData
+{
+  PhotosApplication *application;
+  gchar *extension_name;
+  gchar *miner_name;
+};
+
 struct _PhotosApplicationRefreshData
 {
   PhotosApplication *application;
@@ -129,9 +140,35 @@ struct _PhotosApplicationRefreshData
 
 static gboolean photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner);
 static void photos_application_start_miners (PhotosApplication *self);
+static void photos_application_start_miners_second (PhotosApplication *self);
 static void photos_application_stop_miners (PhotosApplication *self);
 
 
+static PhotosApplicationCreateData *
+photos_application_create_data_new (PhotosApplication *application,
+                                    const gchar *extension_name,
+                                    const gchar *miner_name)
+{
+  PhotosApplicationCreateData *data;
+
+  data = g_slice_new0 (PhotosApplicationCreateData);
+  data->application = g_object_ref (application);
+  data->extension_name = g_strdup (extension_name);
+  data->miner_name = g_strdup (miner_name);
+  return data;
+}
+
+
+static void
+photos_application_create_data_free (PhotosApplicationCreateData *data)
+{
+  g_object_unref (data->application);
+  g_free (data->extension_name);
+  g_free (data->miner_name);
+  g_slice_free (PhotosApplicationCreateData, data);
+}
+
+
 static PhotosApplicationRefreshData *
 photos_application_refresh_data_new (PhotosApplication *application, GomMiner *miner)
 {
@@ -187,13 +224,175 @@ photos_application_action_toggle (GSimpleAction *simple, GVariant *parameter, gp
 }
 
 
+static void
+photos_application_tracker_clear_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  GError *error;
+  TrackerExtractPriority *extract_priority = TRACKER_EXTRACT_PRIORITY (source_object);
+
+  error = NULL;
+  if (!tracker_extract_priority_call_clear_rdf_types_finish (extract_priority, res, &error))
+    {
+      g_warning ("Unable to call ClearRdfTypes: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+ out:
+  g_object_unref (self);
+}
+
+
 static void
 photos_application_destroy (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
 
   priv->main_window = NULL;
+
+  g_cancellable_cancel (priv->create_window_cancellable);
+  g_clear_object (&priv->create_window_cancellable);
+  priv->create_window_cancellable = g_cancellable_new ();
+
   photos_application_stop_miners (self);
+
+  if (priv->extract_priority != NULL)
+    {
+      tracker_extract_priority_call_clear_rdf_types (priv->extract_priority,
+                                                     NULL,
+                                                     photos_application_tracker_clear_rdf_types,
+                                                     g_object_ref (self));
+      g_clear_object (&priv->extract_priority);
+    }
+}
+
+
+static void
+photos_application_gom_miner (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplicationCreateData *data = (PhotosApplicationCreateData *) user_data;
+  PhotosApplication *self = data->application;
+  PhotosApplicationPrivate *priv = self->priv;
+  GError *error;
+  GomMiner *miner = NULL;
+
+  error = NULL;
+  miner = gom_miner_proxy_new_for_bus_finish (res, &error);
+  if (error != NULL)
+    {
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        {
+          g_error_free (error);
+          goto out;
+        }
+      else
+        {
+          g_warning ("Unable to create GomMiner proxy for %s: %s", data->miner_name, error->message);
+          g_error_free (error);
+          goto maybe_continue;
+        }
+    }
+
+  g_object_set_data_full (G_OBJECT (miner), "provider-type", g_strdup (data->extension_name), g_free);
+  priv->miners = g_list_prepend (priv->miners, g_object_ref (miner));
+
+ maybe_continue:
+  if (priv->create_miners_count == 1)
+    photos_application_start_miners_second (self);
+
+ out:
+  priv->create_miners_count--;
+  g_clear_object (&miner);
+  photos_application_create_data_free (data);
+}
+
+
+static void
+photos_application_create_miners (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  GIOExtensionPoint *extension_point;
+  GList *extensions;
+  GList *l;
+
+  extension_point = g_io_extension_point_lookup (PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME);
+  extensions = g_io_extension_point_get_extensions (extension_point);
+  for (l = extensions; l != NULL; l = l->next)
+    {
+      GIOExtension *extension = (GIOExtension *) l->data;
+      PhotosApplicationCreateData *data;
+      PhotosBaseItemClass *base_item_class;
+
+      base_item_class = PHOTOS_BASE_ITEM_CLASS (g_io_extension_ref_class (extension));
+      if (base_item_class->miner_name != NULL && base_item_class->miner_object_path != NULL)
+        {
+          const gchar *extension_name;
+
+          extension_name = g_io_extension_get_name (extension);
+          data = photos_application_create_data_new (self, extension_name, base_item_class->miner_name);
+          gom_miner_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                       G_DBUS_PROXY_FLAGS_NONE,
+                                       base_item_class->miner_name,
+                                       base_item_class->miner_object_path,
+                                       priv->create_window_cancellable,
+                                       photos_application_gom_miner,
+                                       data);
+          priv->create_miners_count++;
+        }
+
+      g_type_class_unref (base_item_class);
+    }
+}
+
+
+static void
+photos_application_tracker_set_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  GError *error;
+  TrackerExtractPriority *extract_priority = TRACKER_EXTRACT_PRIORITY (source_object);
+
+  error = NULL;
+  if (!tracker_extract_priority_call_set_rdf_types_finish (extract_priority, res, &error))
+    {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        g_warning ("Unable to call SetRdfTypes: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+ out:
+  g_object_unref (self);
+}
+
+
+static void
+photos_application_tracker_extract_priority (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosApplicationPrivate *priv = self->priv;
+  GError *error;
+  const gchar *const rdf_types[] = {"nfo:Image", NULL};
+
+  error = NULL;
+  priv->extract_priority = tracker_extract_priority_proxy_new_for_bus_finish (res, &error);
+  if (error != NULL)
+    {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        g_warning ("Unable to create TrackerExtractPriority proxy: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  tracker_extract_priority_call_set_rdf_types (priv->extract_priority,
+                                               rdf_types,
+                                               priv->create_window_cancellable,
+                                               photos_application_tracker_set_rdf_types,
+                                               g_object_ref (self));
+
+ out:
+  g_object_unref (self);
 }
 
 
@@ -207,6 +406,15 @@ photos_application_create_window (PhotosApplication *self)
 
   priv->main_window = photos_main_window_new (GTK_APPLICATION (self));
   g_signal_connect_swapped (priv->main_window, "destroy", G_CALLBACK (photos_application_destroy), self);
+
+  tracker_extract_priority_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                              G_DBUS_PROXY_FLAGS_NONE,
+                                              "org.freedesktop.Tracker1.Miner.Extract",
+                                              "/org/freedesktop/Tracker1/Extract/Priority",
+                                              priv->create_window_cancellable,
+                                              photos_application_tracker_extract_priority,
+                                              g_object_ref (self));
+
   photos_application_start_miners (self);
 }
 
@@ -282,7 +490,7 @@ photos_application_can_fullscreen_changed (PhotosApplication *self)
   PhotosApplicationPrivate *priv = self->priv;
   gboolean can_fullscreen;
 
-  can_fullscreen = photos_mode_controller_get_can_fullscreen (priv->mode_cntrlr);
+  can_fullscreen = photos_mode_controller_get_can_fullscreen (priv->state->mode_cntrlr);
   g_simple_action_set_enabled (priv->fs_action, can_fullscreen);
 }
 
@@ -292,7 +500,7 @@ photos_application_fullscreen (PhotosApplication *self, GVariant *parameter)
 {
   PhotosApplicationPrivate *priv = self->priv;
 
-  photos_mode_controller_toggle_fullscreen (priv->mode_cntrlr);
+  photos_mode_controller_toggle_fullscreen (priv->state->mode_cntrlr);
 }
 
 
@@ -312,7 +520,7 @@ photos_application_launch_search (PhotosApplication *self, const gchar* const *t
   gchar *str;
 
   photos_application_create_window (self);
-  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+  photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
 
   str = g_strjoinv (" ", (gchar **) terms);
   photos_search_controller_set_string (priv->state->srch_cntrlr, str);
@@ -534,21 +742,28 @@ photos_application_set_bg_common (PhotosApplication *self, GVariant *parameter,
 
 static void
 photos_application_start_miners (PhotosApplication *self)
+{
+  photos_application_create_miners (self);
+}
+
+
+static void
+photos_application_start_miners_second (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
 
   photos_application_refresh_miners (self);
 
-  g_signal_connect_object (priv->state->src_mngr,
-                           "object-added",
-                           G_CALLBACK (photos_application_refresh_miners),
-                           self,
-                           G_CONNECT_SWAPPED);
-  g_signal_connect_object (priv->state->src_mngr,
-                           "object-removed",
-                           G_CALLBACK (photos_application_refresh_miners),
-                           self,
-                           G_CONNECT_SWAPPED);
+  priv->source_added_id = g_signal_connect_object (priv->state->src_mngr,
+                                                   "object-added",
+                                                   G_CALLBACK (photos_application_refresh_miners),
+                                                   self,
+                                                   G_CONNECT_SWAPPED);
+  priv->source_removed_id = g_signal_connect_object (priv->state->src_mngr,
+                                                     "object-removed",
+                                                     G_CALLBACK (photos_application_refresh_miners),
+                                                     self,
+                                                     G_CONNECT_SWAPPED);
 }
 
 
@@ -566,6 +781,21 @@ photos_application_stop_miners (PhotosApplication *self)
       cancellable = g_object_get_data (G_OBJECT (miner), "cancellable");
       g_cancellable_cancel (cancellable);
     }
+
+  if (priv->source_added_id != 0)
+    {
+      g_signal_handler_disconnect (priv->state->src_mngr, priv->source_added_id);
+      priv->source_added_id = 0;
+    }
+
+  if (priv->source_removed_id != 0)
+    {
+      g_signal_handler_disconnect (priv->state->src_mngr, priv->source_removed_id);
+      priv->source_removed_id = 0;
+    }
+
+  g_list_free_full (priv->miners, g_object_unref);
+  priv->miners = NULL;
 }
 
 
@@ -605,72 +835,6 @@ photos_application_theme_changed (GtkSettings *settings)
 }
 
 
-static void
-photos_application_tracker_clear_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
-  GError *error;
-
-  error = NULL;
-  if (!tracker_extract_priority_call_clear_rdf_types_finish (self->priv->extract_priority, res, &error))
-    {
-      g_warning ("Unable to call ClearRdfTypes: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
- out:
-  g_object_unref (self);
-}
-
-
-static void
-photos_application_tracker_set_rdf_types (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
-  GError *error;
-
-  error = NULL;
-  if (!tracker_extract_priority_call_set_rdf_types_finish (self->priv->extract_priority, res, &error))
-    {
-      g_warning ("Unable to call SetRdfTypes: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
- out:
-  g_object_unref (self);
-}
-
-
-static void
-photos_application_tracker_extract_priority (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
-  PhotosApplicationPrivate *priv = self->priv;
-  GError *error;
-  const gchar *const rdf_types[] = {"nfo:Image", NULL};
-
-  error = NULL;
-  priv->extract_priority = tracker_extract_priority_proxy_new_for_bus_finish (res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to create TrackerExtractPriority proxy: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
-  tracker_extract_priority_call_set_rdf_types (priv->extract_priority,
-                                               rdf_types,
-                                               NULL,
-                                               photos_application_tracker_set_rdf_types,
-                                               g_object_ref (self));
-
- out:
-  g_object_unref (self);
-}
-
-
 static void
 photos_application_window_mode_changed (PhotosApplication *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
 {
@@ -693,6 +857,8 @@ photos_application_window_mode_changed (PhotosApplication *self, PhotosWindowMod
 
   enable = (mode == PHOTOS_WINDOW_MODE_PREVIEW);
   g_simple_action_set_enabled (priv->gear_action, enable);
+  g_simple_action_set_enabled (priv->load_next_action, enable);
+  g_simple_action_set_enabled (priv->load_previous_action, enable);
   g_simple_action_set_enabled (priv->open_action, enable);
   g_simple_action_set_enabled (priv->print_action, enable);
   g_simple_action_set_enabled (priv->properties_action, enable);
@@ -710,7 +876,7 @@ photos_application_activate (GApplication *application)
   if (priv->main_window == NULL)
     {
       photos_application_create_window (self);
-      photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+      photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
     }
 
   gtk_window_present_with_time (GTK_WINDOW (priv->main_window), priv->activation_timestamp);
@@ -775,13 +941,7 @@ photos_application_dbus_unregister (GApplication *application,
 static void
 photos_application_shutdown (GApplication *application)
 {
-  PhotosApplication *self = PHOTOS_APPLICATION (application);
-
   gegl_exit ();
-  tracker_extract_priority_call_clear_rdf_types (self->priv->extract_priority,
-                                                 NULL,
-                                                 photos_application_tracker_clear_rdf_types,
-                                                 g_object_ref (self));
 
   G_APPLICATION_CLASS (photos_application_parent_class)->shutdown (application);
 }
@@ -793,14 +953,12 @@ photos_application_startup (GApplication *application)
   PhotosApplication *self = PHOTOS_APPLICATION (application);
   PhotosApplicationPrivate *priv = self->priv;
   GError *error;
-  GIOExtensionPoint *extension_point;
-  GList *extensions;
-  GList *l;
   GSimpleAction *action;
   GrlRegistry *registry;
   GtkSettings *settings;
   GVariant *state;
   GVariantType *parameter_type;
+  const gchar *delete_accels[2] = {"Delete", "KP_Delete", NULL};
   const gchar *fullscreen_accels[2] = {"F11", NULL};
   const gchar *gear_menu_accels[2] = {"F10", NULL};
   const gchar *print_current_accels[2] = {"<Primary>p", NULL};
@@ -821,6 +979,8 @@ photos_application_startup (GApplication *application)
       g_error_free (error);
     }
 
+  priv->create_window_cancellable = g_cancellable_new ();
+
   priv->bg_settings = g_settings_new (DESKTOP_BACKGROUND_SCHEMA);
   priv->ss_settings = g_settings_new (DESKTOP_SCREENSAVER_SCHEMA);
 
@@ -832,68 +992,25 @@ photos_application_startup (GApplication *application)
   g_signal_connect (settings, "notify::gtk-theme-name", G_CALLBACK (photos_application_theme_changed), NULL);
   photos_application_theme_changed (settings);
 
-  extension_point = g_io_extension_point_lookup (PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME);
-  extensions = g_io_extension_point_get_extensions (extension_point);
-  for (l = extensions; l != NULL; l = l->next)
-    {
-      GIOExtension *extension = (GIOExtension *) l->data;
-      PhotosBaseItemClass *base_item_class;
-
-      base_item_class = PHOTOS_BASE_ITEM_CLASS (g_io_extension_ref_class (extension));
-      if (base_item_class->miner_name != NULL && base_item_class->miner_object_path != NULL)
-        {
-          GomMiner *miner;
-          const gchar *extension_name;
-
-          error = NULL;
-          miner = gom_miner_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
-                                                    G_DBUS_PROXY_FLAGS_NONE,
-                                                    base_item_class->miner_name,
-                                                    base_item_class->miner_object_path,
-                                                    NULL,
-                                                    &error);
-          if (error != NULL)
-            {
-              g_warning ("Unable to create GomMiner proxy for %s: %s", base_item_class->miner_name, error->message);
-              g_error_free (error);
-              continue;
-            }
-
-          extension_name = g_io_extension_get_name (extension);
-          g_object_set_data_full (G_OBJECT (miner), "provider-type", g_strdup (extension_name), g_free);
-
-          priv->miners = g_list_prepend (priv->miners, g_object_ref (miner));
-          g_object_unref (miner);
-        }
-
-      g_type_class_unref (base_item_class);
-    }
-
-  tracker_extract_priority_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-                                              G_DBUS_PROXY_FLAGS_NONE,
-                                              "org.freedesktop.Tracker1.Miner.Extract",
-                                              "/org/freedesktop/Tracker1/Extract/Priority",
-                                              NULL,
-                                              photos_application_tracker_extract_priority,
-                                              g_object_ref (self));
-
   /* A dummy reference to keep it alive during the lifetime of the
    * application.
    */
   priv->camera_cache = photos_camera_cache_dup_singleton ();
 
-  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
-
   action = g_simple_action_new ("about", NULL);
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_about), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  action = g_simple_action_new ("delete", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
+  g_object_unref (action);
+
   priv->fs_action = g_simple_action_new ("fullscreen", NULL);
   g_signal_connect_swapped (priv->fs_action, "activate", G_CALLBACK (photos_application_fullscreen), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->fs_action));
 
-  g_signal_connect_swapped (priv->mode_cntrlr,
+  g_signal_connect_swapped (priv->state->mode_cntrlr,
                             "can-fullscreen-changed",
                             G_CALLBACK (photos_application_can_fullscreen_changed),
                             self);
@@ -903,6 +1020,12 @@ photos_application_startup (GApplication *application)
   g_signal_connect (priv->gear_action, "activate", G_CALLBACK (photos_application_action_toggle), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->gear_action));
 
+  priv->load_next_action = g_simple_action_new ("load-next", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->load_next_action));
+
+  priv->load_previous_action = g_simple_action_new ("load-previous", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->load_previous_action));
+
   priv->open_action = g_simple_action_new ("open-current", NULL);
   g_signal_connect_swapped (priv->open_action, "activate", G_CALLBACK (photos_application_open_current), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->open_action));
@@ -920,12 +1043,10 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
-  priv->remote_display_action = g_simple_action_new ("remote-display-current", NULL);
-  g_signal_connect_swapped (priv->remote_display_action,
-                            "activate",
-                            G_CALLBACK (photos_application_remote_display_current),
-                            self);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->remote_display_action));
+  action = g_simple_action_new ("remote-display-current", NULL);
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_remote_display_current), self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
+  g_object_unref (action);
 
   state = g_variant_new ("b", FALSE);
   priv->search_action = g_simple_action_new_stateful ("search", NULL, state);
@@ -972,7 +1093,7 @@ photos_application_startup (GApplication *application)
   g_signal_connect_swapped (priv->set_ss_action, "activate", G_CALLBACK (photos_application_set_bg_common), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->set_ss_action));
 
-  g_signal_connect_swapped (priv->mode_cntrlr,
+  g_signal_connect_swapped (priv->state->mode_cntrlr,
                             "window-mode-changed",
                             G_CALLBACK (photos_application_window_mode_changed),
                             self);
@@ -983,6 +1104,7 @@ photos_application_startup (GApplication *application)
   g_object_unref (action);
 
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.quit", quit_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.delete", delete_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.fullscreen", fullscreen_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.gear-menu", gear_menu_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.print-current", print_current_accels);
@@ -1034,10 +1156,13 @@ photos_application_dispose (GObject *object)
       priv->resource = NULL;
     }
 
+  g_clear_object (&priv->create_window_cancellable);
   g_clear_object (&priv->bg_settings);
   g_clear_object (&priv->ss_settings);
   g_clear_object (&priv->fs_action);
   g_clear_object (&priv->gear_action);
+  g_clear_object (&priv->load_next_action);
+  g_clear_object (&priv->load_previous_action);
   g_clear_object (&priv->open_action);
   g_clear_object (&priv->print_action);
   g_clear_object (&priv->properties_action);
@@ -1050,7 +1175,6 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->set_bg_action);
   g_clear_object (&priv->set_ss_action);
   g_clear_object (&priv->camera_cache);
-  g_clear_object (&priv->mode_cntrlr);
   g_clear_object (&priv->extract_priority);
 
   if (priv->state != NULL)
@@ -1063,6 +1187,20 @@ photos_application_dispose (GObject *object)
 }
 
 
+static void
+photos_application_finalize (GObject *object)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (object);
+  PhotosApplicationPrivate *priv = self->priv;
+
+  if (G_UNLIKELY (priv->create_miners_count != 0))
+    g_critical ("Application is being destroyed while %u miner proxies are still being created.",
+                priv->create_miners_count);
+
+  G_OBJECT_CLASS (photos_application_parent_class)->finalize (object);
+}
+
+
 static void
 photos_application_init (PhotosApplication *self)
 {
@@ -1098,6 +1236,7 @@ photos_application_class_init (PhotosApplicationClass *class)
 
   object_class->constructor = photos_application_constructor;
   object_class->dispose = photos_application_dispose;
+  object_class->finalize = photos_application_finalize;
   application_class->activate = photos_application_activate;
   application_class->dbus_register = photos_application_dbus_register;
   application_class->dbus_unregister = photos_application_dbus_unregister;
