@@ -36,7 +36,6 @@
 #include "photos-icons.h"
 #include "photos-item-manager.h"
 #include "photos-main-toolbar.h"
-#include "photos-mode-controller.h"
 #include "photos-overview-searchbar.h"
 #include "photos-remote-display-manager.h"
 #include "photos-search-context.h"
@@ -167,6 +166,64 @@ photos_main_toolbar_add_back_button (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_remote_display_button_clicked (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+
+  photos_remote_display_manager_stop (priv->remote_mngr);
+}
+
+
+static void
+photos_main_toolbar_add_remote_display_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  PhotosDlnaRenderer *renderer;
+  GtkLabel *label;
+  gchar *text;
+  const gchar *name;
+
+  g_clear_pointer (&priv->remote_display_button, (GDestroyNotify) gtk_widget_destroy);
+
+  renderer = photos_remote_display_manager_get_renderer (priv->remote_mngr);
+  name = photos_dlna_renderer_get_friendly_name (renderer);
+  text = g_markup_printf_escaped ("Displaying on <b>%s</b>", name);
+
+  priv->remote_display_button = gtk_button_new_with_label (text);
+  label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (priv->remote_display_button)));
+  gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
+  gtk_label_set_use_markup (label, TRUE);
+  gtk_widget_set_margin_end (priv->remote_display_button, 12);
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->toolbar), priv->remote_display_button);
+  gtk_widget_show_all (priv->remote_display_button);
+
+  g_signal_connect_swapped (priv->remote_display_button,
+                            "clicked",
+                            G_CALLBACK (photos_main_toolbar_remote_display_button_clicked),
+                            self);
+  g_free (text);
+}
+
+
+static void
+photos_main_toolbar_update_remote_display_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  PhotosWindowMode window_mode;
+  gboolean selection_mode, active;
+
+  selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
+  window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  active = photos_remote_display_manager_is_active (priv->remote_mngr);
+
+  if (active && !selection_mode && window_mode != PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_main_toolbar_add_remote_display_button (self);
+  else
+    g_clear_pointer (&priv->remote_display_button, (GDestroyNotify) gtk_widget_destroy);
+}
+
+
 static void
 photos_main_toolbar_coll_back_button_clicked (PhotosMainToolbar *self)
 {
@@ -197,14 +254,11 @@ photos_main_toolbar_col_active_changed (PhotosMainToolbar *self, PhotosBaseItem
   else
     {
       mode = PHOTOS_HEADER_BAR_MODE_NORMAL;
-      if (priv->coll_back_button != NULL)
-        {
-          gtk_widget_destroy (priv->coll_back_button);
-          priv->coll_back_button = NULL;
-        }
+      g_clear_pointer (&priv->coll_back_button, (GDestroyNotify) gtk_widget_destroy);
     }
 
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), mode);
+  photos_main_toolbar_update_remote_display_button (self);
   photos_main_toolbar_set_toolbar_title (self);
 }
 
@@ -237,47 +291,6 @@ photos_main_toolbar_select_button_clicked (PhotosMainToolbar *self)
 }
 
 
-static void
-photos_main_toolbar_remote_display_button_clicked (PhotosMainToolbar *self)
-{
-  PhotosMainToolbarPrivate *priv = self->priv;
-
-  photos_remote_display_manager_stop (priv->remote_mngr);
-}
-
-
-static void
-photos_main_toolbar_add_remote_display_button (PhotosMainToolbar *self)
-{
-  PhotosMainToolbarPrivate *priv = self->priv;
-  PhotosDlnaRenderer *renderer;
-  GtkLabel *label;
-  gchar *text;
-  const gchar *name;
-
-  if (priv->remote_display_button != NULL)
-    gtk_widget_destroy (priv->remote_display_button);
-
-  renderer = photos_remote_display_manager_get_renderer (priv->remote_mngr);
-  name = photos_dlna_renderer_get_friendly_name (renderer);
-  text = g_markup_printf_escaped ("Displaying on <b>%s</b>", name);
-
-  priv->remote_display_button = gtk_button_new_with_label (text);
-  label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (priv->remote_display_button)));
-  gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
-  gtk_label_set_use_markup (label, TRUE);
-  gtk_widget_set_margin_end (priv->remote_display_button, 12);
-  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->toolbar), priv->remote_display_button);
-  gtk_widget_show_all (priv->remote_display_button);
-
-  g_signal_connect_swapped (priv->remote_display_button,
-                            "clicked",
-                            G_CALLBACK (photos_main_toolbar_remote_display_button_clicked),
-                            self);
-  g_free (text);
-}
-
-
 static GtkWidget *
 photos_main_toolbar_add_search_button (PhotosMainToolbar *self)
 {
@@ -327,7 +340,6 @@ photos_main_toolbar_back_button_clicked (PhotosMainToolbar *self)
   if (priv->loader_cancellable != NULL)
     g_cancellable_cancel (priv->loader_cancellable);
 
-  photos_base_manager_set_active_object (priv->item_mngr, NULL);
   photos_mode_controller_go_back (priv->mode_cntrlr);
 }
 
@@ -337,23 +349,9 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
 
-  if (priv->coll_back_button != NULL)
-    {
-      gtk_widget_destroy (priv->coll_back_button);
-      priv->coll_back_button = NULL;
-    }
-
-  if (priv->remote_display_button != NULL)
-    {
-      gtk_widget_destroy (priv->remote_display_button);
-      priv->remote_display_button = NULL;
-    }
-
-  if (priv->selection_button != NULL)
-    {
-      gtk_widget_destroy (priv->selection_button);
-      priv->selection_button = NULL;
-    }
+  g_clear_pointer (&priv->coll_back_button, (GDestroyNotify) gtk_widget_destroy);
+  g_clear_pointer (&priv->remote_display_button, (GDestroyNotify) gtk_widget_destroy);
+  g_clear_pointer (&priv->selection_button, (GDestroyNotify) gtk_widget_destroy);
 
   if (priv->searchbar != NULL && gtk_widget_get_parent (priv->searchbar) == GTK_WIDGET (self))
     {
@@ -457,26 +455,21 @@ static void
 photos_main_toolbar_favorite_button_update (PhotosMainToolbar *self, gboolean favorite)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GtkStyleContext *context;
+  GtkWidget *image;
   gchar *favorite_label;
 
-  g_signal_handlers_block_by_func (priv->favorite_button, photos_main_toolbar_favorite_button_clicked, self);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->favorite_button), favorite);
-  g_signal_handlers_unblock_by_func (priv->favorite_button, photos_main_toolbar_favorite_button_clicked, self);
-
-  context = gtk_widget_get_style_context (priv->favorite_button);
   if (favorite)
     {
       favorite_label = g_strdup (_("Remove from favorites"));
-      gtk_style_context_add_class (context, "photos-favorite");
+      image = gtk_image_new_from_icon_name (PHOTOS_ICON_FAVORITE_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
     }
   else
     {
       favorite_label = g_strdup (_("Add to favorites"));
-      gtk_style_context_remove_class (context, "photos-favorite");
+      image = gtk_image_new_from_icon_name (PHOTOS_ICON_NOT_FAVORITE_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
     }
 
-  gtk_widget_reset_style (priv->favorite_button);
+  gtk_button_set_image (GTK_BUTTON (priv->favorite_button), image);
   gtk_widget_set_tooltip_text (priv->favorite_button, favorite_label);
   g_free (favorite_label);
 }
@@ -501,24 +494,6 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 }
 
 
-static void
-photos_main_toolbar_update_remote_display_button (PhotosMainToolbar *self)
-{
-  PhotosMainToolbarPrivate *priv = self->priv;
-  PhotosWindowMode window_mode;
-  gboolean selection_mode, active;
-
-  selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
-  window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
-  active = photos_remote_display_manager_is_active (priv->remote_mngr);
-
-  if (active && !selection_mode && window_mode != PHOTOS_WINDOW_MODE_PREVIEW)
-    photos_main_toolbar_add_remote_display_button (self);
-  else
-    g_clear_pointer (&priv->remote_display_button, gtk_widget_destroy);
-}
-
-
 static void
 photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
@@ -587,9 +562,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 
   g_simple_action_set_enabled (priv->gear_menu, TRUE);
 
-  image = gtk_image_new_from_icon_name (PHOTOS_ICON_FAVORITE_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
-  priv->favorite_button = gtk_toggle_button_new ();
-  gtk_button_set_image (GTK_BUTTON (priv->favorite_button), image);
+  priv->favorite_button = gtk_button_new ();
   gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), priv->favorite_button);
   g_signal_connect_swapped (priv->favorite_button,
                             "clicked",
@@ -774,7 +747,7 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
 
   priv->item_mngr = g_object_ref (state->item_mngr);
 
-  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  priv->mode_cntrlr = g_object_ref (state->mode_cntrlr);
 
   priv->sel_cntrlr = photos_selection_controller_dup_singleton ();
   g_signal_connect_object (priv->sel_cntrlr,
