@@ -33,7 +33,6 @@
 #include "photos-embed.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
-#include "photos-mode-controller.h"
 #include "photos-preview-view.h"
 #include "photos-search-context.h"
 #include "photos-selection-controller.h"
@@ -42,6 +41,8 @@
 
 struct _PhotosMainWindowPrivate
 {
+  GAction *load_next;
+  GAction *load_previous;
   GtkWidget *embed;
   GSettings *settings;
   PhotosBaseManager *item_mngr;
@@ -163,7 +164,6 @@ photos_main_window_go_back (PhotosMainWindow *self)
   switch (mode)
     {
     case PHOTOS_WINDOW_MODE_PREVIEW:
-      photos_base_manager_set_active_object (priv->item_mngr, NULL);
       photos_mode_controller_go_back (priv->mode_cntrlr);
       break;
 
@@ -231,30 +231,25 @@ static gboolean
 photos_main_window_handle_key_preview (PhotosMainWindow *self, GdkEventKey *event)
 {
   PhotosMainWindowPrivate *priv = self->priv;
-  PhotosPreviewView *preview;
   gboolean fullscreen;
   gboolean handled = FALSE;
 
-  preview = photos_embed_get_preview (PHOTOS_EMBED (priv->embed));
   fullscreen = photos_mode_controller_get_fullscreen (priv->mode_cntrlr);
 
   switch (event->keyval)
     {
     case GDK_KEY_Escape:
       if (fullscreen)
-        {
-          photos_base_manager_set_active_object (priv->item_mngr, NULL);
-          photos_mode_controller_go_back (priv->mode_cntrlr);
-        }
+        photos_mode_controller_go_back (priv->mode_cntrlr);
       break;
 
     case GDK_KEY_Left:
-      photos_preview_view_load_previous (preview);
+      g_action_activate (priv->load_previous, NULL);
       handled = TRUE;
       break;
 
     case GDK_KEY_Right:
-      photos_preview_view_load_next (preview);
+      g_action_activate (priv->load_next, NULL);
       handled = TRUE;
       break;
 
@@ -365,6 +360,9 @@ photos_main_window_constructed (GObject *object)
   app = g_application_get_default ();
   gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (self));
 
+  priv->load_next = g_action_map_lookup_action (G_ACTION_MAP (app), "load-next");
+  priv->load_previous = g_action_map_lookup_action (G_ACTION_MAP (app), "load-previous");
+
   priv->embed = photos_embed_new ();
   gtk_container_add (GTK_CONTAINER (self), priv->embed);
 }
@@ -429,7 +427,7 @@ photos_main_window_init (PhotosMainWindow *self)
 
   priv->item_mngr = g_object_ref (state->item_mngr);
 
-  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  priv->mode_cntrlr = g_object_ref (state->mode_cntrlr);
   g_signal_connect_swapped (priv->mode_cntrlr,
                             "fullscreen-changed",
                             G_CALLBACK (photos_main_window_fullscreen_changed),
@@ -479,7 +477,7 @@ photos_main_window_show_about (PhotosMainWindow *self)
                          "comments", _("Access, organize and share your photos on GNOME"),
                          "copyright", _("Copyright © 2013 Intel Corporation. All rights reserved.\n"
                                         "Copyright © 2014, 2015 Pranav Kant\n"
-                                        "Copyright © 2012, 2013, 2014 Red Hat, Inc."),
+                                        "Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc."),
                          "license-type", GTK_LICENSE_GPL_2_0,
                          "logo-icon-name", PACKAGE_TARNAME,
                          "program-name", _(PACKAGE_NAME),
