@@ -26,12 +26,14 @@
 #include "config.h"
 
 #include <glib.h>
+#include <gio/gio.h>
 #include <glib/gi18n.h>
 
 #include "gegl-gtk-view.h"
-#include "photos-mode-controller.h"
+#include "photos-item-manager.h"
 #include "photos-preview-nav-buttons.h"
 #include "photos-preview-view.h"
+#include "photos-search-context.h"
 
 
 struct _PhotosPreviewViewPrivate
@@ -101,32 +103,12 @@ photos_preview_view_get_invisible_view (PhotosPreviewView *self)
 
 
 static void
-photos_preview_view_nav_buttons_activated (PhotosPreviewView *self, PhotosPreviewAction action)
+photos_preview_view_navigate (PhotosPreviewView *self, gint position)
 {
   PhotosPreviewViewPrivate *priv = self->priv;
   GtkWidget *current_view;
   GtkWidget *new_view;
   GtkWidget *next_view;
-  gint position;
-
-  if (action == PHOTOS_PREVIEW_ACTION_NONE)
-    return;
-
-  switch (action)
-    {
-    case PHOTOS_PREVIEW_ACTION_NEXT:
-      position = 0;
-      break;
-
-    case PHOTOS_PREVIEW_ACTION_PREVIOUS:
-      position = -1;
-      break;
-
-    case PHOTOS_PREVIEW_ACTION_NONE:
-    default:
-      g_assert_not_reached ();
-      break;
-    }
 
   current_view = gtk_stack_get_visible_child (GTK_STACK (priv->stack));
   gtk_container_child_set (GTK_CONTAINER (priv->stack), current_view, "position", position, NULL);
@@ -141,6 +123,20 @@ photos_preview_view_nav_buttons_activated (PhotosPreviewView *self, PhotosPrevie
 }
 
 
+static void
+photos_preview_view_navigate_next (PhotosPreviewView *self)
+{
+  photos_preview_view_navigate (self, 0);
+}
+
+
+static void
+photos_preview_view_navigate_previous (PhotosPreviewView *self)
+{
+  photos_preview_view_navigate (self, -1);
+}
+
+
 static GtkWidget *
 photos_preview_view_create_view (PhotosPreviewView *self)
 {
@@ -164,13 +160,16 @@ photos_preview_view_create_view (PhotosPreviewView *self)
 static void
 photos_preview_view_window_mode_changed (PhotosPreviewView *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
 {
+  PhotosPreviewViewPrivate *priv = self->priv;
+
   switch (mode)
     {
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
     case PHOTOS_WINDOW_MODE_FAVORITES:
     case PHOTOS_WINDOW_MODE_OVERVIEW:
     case PHOTOS_WINDOW_MODE_SEARCH:
-      photos_preview_nav_buttons_hide (self->priv->nav_buttons);
+      photos_preview_nav_buttons_hide (priv->nav_buttons);
+      photos_preview_nav_buttons_set_model (priv->nav_buttons, NULL, NULL);
       break;
 
     case PHOTOS_WINDOW_MODE_PREVIEW:
@@ -211,11 +210,6 @@ photos_preview_view_constructed (GObject *object)
   gtk_container_add (GTK_CONTAINER (self), priv->stack);
 
   priv->nav_buttons = photos_preview_nav_buttons_new (self, GTK_OVERLAY (priv->overlay));
-  g_signal_connect_swapped (priv->nav_buttons,
-                            "activated",
-                            G_CALLBACK (photos_preview_view_nav_buttons_activated),
-                            self);
-
   gtk_widget_show_all (GTK_WIDGET (self));
 }
 
@@ -243,13 +237,19 @@ static void
 photos_preview_view_init (PhotosPreviewView *self)
 {
   PhotosPreviewViewPrivate *priv;
+  GAction *action;
+  GApplication *app;
   GtkStyleContext *context;
   GtkWidget *view;
+  PhotosSearchContextState *state;
 
   self->priv = photos_preview_view_get_instance_private (self);
   priv = self->priv;
 
-  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  app = g_application_get_default ();
+  state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
+
+  priv->mode_cntrlr = g_object_ref (state->mode_cntrlr);
   g_signal_connect_object (priv->mode_cntrlr,
                            "window-mode-changed",
                            G_CALLBACK (photos_preview_view_window_mode_changed),
@@ -270,6 +270,12 @@ photos_preview_view_init (PhotosPreviewView *self)
 
   view = photos_preview_view_create_view (self);
   gtk_container_add (GTK_CONTAINER (priv->stack), view);
+
+  action = g_action_map_lookup_action (G_ACTION_MAP (app), "load-next");
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_preview_view_navigate_next), self);
+
+  action = g_action_map_lookup_action (G_ACTION_MAP (app), "load-previous");
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_preview_view_navigate_previous), self);
 }
 
 
@@ -299,20 +305,6 @@ photos_preview_view_new (GtkOverlay *overlay)
 }
 
 
-void
-photos_preview_view_load_next (PhotosPreviewView *self)
-{
-  photos_preview_nav_buttons_next (self->priv->nav_buttons);
-}
-
-
-void
-photos_preview_view_load_previous (PhotosPreviewView *self)
-{
-  photos_preview_nav_buttons_previous (self->priv->nav_buttons);
-}
-
-
 void
 photos_preview_view_set_model (PhotosPreviewView *self, GtkTreeModel *model, GtkTreePath *current_path)
 {
