@@ -57,7 +57,7 @@ GIcon           *photos_utils_create_collection_icon      (gint base_size, GList
 
 GdkPixbuf       *photos_utils_create_pixbuf_from_node     (GeglNode *node);
 
-GIcon           *photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale);
+GIcon           *photos_utils_create_symbolic_icon        (const gchar *name, gint base_size);
 
 gboolean         photos_utils_create_thumbnail            (GFile *file, GCancellable *cancellable, GError **error);
 
@@ -101,8 +101,6 @@ GList           *photos_utils_get_urns_from_paths         (GList *paths, GtkTree
 
 GIcon           *photos_utils_icon_from_rdf_type          (const gchar *type);
 
-gboolean         photos_utils_queue_thumbnail_job_for_file_finish (GAsyncResult *res);
-
 void             photos_utils_set_edited_name             (const gchar *urn, const gchar *title);
 
 void             photos_utils_set_favorite                (const gchar *urn, gboolean is_favorite);
