@@ -302,7 +302,7 @@ photos_view_container_constructed (GObject *object)
                                                 photos_selection_controller_get_selection_mode (priv->sel_cntrlr),
                                                 self);
 
-  priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
+  priv->mode_cntrlr = g_object_ref (state->mode_cntrlr);
   g_signal_connect_object (priv->mode_cntrlr,
                            "window-mode-changed",
                            G_CALLBACK (photos_view_container_window_mode_changed),
@@ -341,18 +341,10 @@ photos_view_container_constructed (GObject *object)
     }
 
   action = g_action_map_lookup_action (G_ACTION_MAP (app), "select-all");
-  g_signal_connect_object (action,
-                           "activate",
-                           G_CALLBACK (photos_view_container_select_all),
-                           self,
-                           G_CONNECT_SWAPPED);
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_view_container_select_all), self);
 
   action = g_action_map_lookup_action (G_ACTION_MAP (app), "select-none");
-  g_signal_connect_object (action,
-                           "activate",
-                           G_CALLBACK (gd_main_view_unselect_all),
-                           priv->view,
-                           G_CONNECT_SWAPPED);
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (gd_main_view_unselect_all), priv->view);
 
   g_signal_connect_object (priv->offset_cntrlr,
                            "count-changed",
